`timescale 10ps / 10ps
`define GTK // use VCD

`define MODULE	tb_counter
`define CLK_LIMIT	24'h400 // for test

`ifdef CLK_LIMIT
`else
`define CLK_LIMIT	24'h100000
`endif

module `MODULE;

   reg [23:0] CLK_COUNT;
   reg CLK;   // input
   wire [7:0] A;
   wire [7:0] B;
   wire [7:0] F;
   wire [2:0] S;
   wire [1:0] ES;
   wire C_in;
   wire Z_in;
   wire C_out;
   wire H_out;
   wire V_out;
   wire Z_out;
   assign Z_in = 1'b0;
   assign ES = 2'b00;
   assign C_in = CLK_COUNT[0];
   assign S = CLK_COUNT[3:1];
   assign B[7:0] = { CLK_COUNT[8], CLK_COUNT[14:10] , CLK_COUNT[5:4] };
   assign A[7:0] = { CLK_COUNT[9], CLK_COUNT[19:15] , CLK_COUNT[7:6] };

rtavr_alu top
	( .ES(ES[1:0]), .S(S[2:0]), .A(A[7:0]) , .B(B[7:0]) , .F(F[7:0])
	  , .C_in(C_in), .Z_in(Z_in), .C_out(C_out)
	  , .H_out(H_out), .V_out(V_out), .Z_out(Z_out) );

   parameter CLKP = 5000000;        //  50us
   parameter CLKH = CLKP/2;      //
   parameter CLKL = CLKP-CLKH;   //
   
   always begin
      #CLKH CLK = 0;
      #CLKL CLK = 1;
   end

`ifdef GTK
   initial begin
      $dumpfile("tb.vcd");
      $dumpvars(0,`MODULE.top);
   end
`endif

   integer fp;
   initial begin
      fp = $fopen("tb.log");
   end

   always @(posedge CLK)
   begin
     if (~S[2])
        $fdisplay(fp, "S %01x : A %02x B %02x C %d : F %02x CHVZ %d%d%d%d "
		, S[2:0]
		, A[7:0] , B[7:0] , C_in
		, F[7:0] , C_out , H_out, V_out , Z_out );
     else
        $fdisplay(fp, "S %01x : A %02x B %02x C %d : F %02x CHVZ xx%d%d "
		, S[2:0]
		, A[7:0] , B[7:0] , C_in
		, F[7:0] , V_out , Z_out );
   end

   initial begin : test
      CLK_COUNT = 0;
      //I = 1'b0;
      //R = 1'b0;
      //sys_reset();
      while ( (CLK_COUNT < `CLK_LIMIT) )
      begin
         @(posedge CLK);
         CLK_COUNT = CLK_COUNT + 1;
      end
//      @(posedge CLK);
      #1
      $display("*** Simulation Complete! ***");
      $finish;
   end

   task sys_reset;
      begin
         //PC2 = 0;
         //#1 RESET = 1;
         //CLK2X = 0;
         @(posedge CLK);
         @(posedge CLK);
         //#1 RESET = 0;
         @(posedge CLK);
         @(posedge CLK);
         CLK_COUNT = CLK_COUNT + 1;
      end
   endtask // sys_reset
   
endmodule // test
